@@ -146,7 +146,7 @@ long CodonAlignStringsStep( double * const score_matrix
          best_choice = 0,
          i, choice, partial_codons[ 10 ]; // we need to multiply by 3 to get the NUC position
     // 3x5 codon specifications (negative indices)
-    long codon_spec_3x5[ 10 ][ 3 ] = {
+    static long const codon_spec_3x5[ 10 ][ 3 ] = {
         { 5, 4, 3 }, // 11100
         { 5, 4, 2 }, // 11010
         { 5, 4, 1 }, // 11001
@@ -159,7 +159,7 @@ long CodonAlignStringsStep( double * const score_matrix
         { 3, 2, 1 }  // 00111
     };
     // 3x4 codon specifications (negative indices)
-    long codon_spec_3x4[ 4 ][ 3 ] = {
+    static long const long codon_spec_3x4[ 4 ][ 3 ] = {
         { 4, 3, 2 }, // 1110
         { 4, 3, 1 }, // 1101
         { 4, 2, 1 }, // 1011
@@ -388,16 +388,23 @@ long CodonAlignStringsStep( double * const score_matrix
     }
 
     // find the best possible choice
-    for ( i = 0; i < HY_ALIGNMENT_TYPES_COUNT - (!do_local); ++i ) {
-        /* if ( i > 0 )
-         fprintf( stderr, ", " );
-         fprintf( stderr, "( %ld, %.3g )", i, choices[ i ] ); */
-        if ( choices[ i ] > max_score ) {
-            best_choice = i;
-            max_score = choices[ i ];
+    if (do_local) {
+        for ( i = 0; i < HY_ALIGNMENT_TYPES_COUNT ; ++i ) {
+            if ( choices[ i ] > max_score ) {
+                best_choice = i;
+                max_score = choices[ i ];
+            }
+        }
+    } else {
+        for ( i = 0; i < HY_ALIGNMENT_TYPES_COUNT - 1 ; ++i ) {
+            if ( choices[ i ] > max_score ) {
+                best_choice = i;
+                max_score = choices[ i ];
+            }
         }
     }
     
+    
     //fprintf( stderr, "\nscore: %.3g best: %ld\n", max_score, best_choice );
 
     // assign the score to the current position
@@ -987,6 +994,7 @@ double AlignStrings( char const * r_str
 
                     // if anything drops below 0, something bad happened
                     if ( i < 0 || j < 0 ) {
+                        delete [] edit_ops;
                         return -INFINITY;
                     }
 
