@@ -98,6 +98,22 @@ protected:
     // actual storage allocated
     
     long*       theIndex;                       // indices of matrix elements in logical storage
+    long*       compressedIndex;
+        /**
+            20200821 SLKP to speed sparse caclulations CompressSparseMatrix will create this DENSE index, which has the following structure
+                - Entries in theIndex are expected to be compressed (no -1) and arranged BY ROW
+                - First hDim values: the INDEX of the non-first non zero index for this ROW in theData
+                - Next lDim values: the COLUMN index for the corresponding entry
+         
+                [x,1,2,x]
+                [1,2,x,x]
+                [x,x,2,x]
+                [x,x,x,3]
+         
+                theIndex : [1,2,4,5,10,15]
+                compressedIndex: [0,2,4,5,1,2,0,1,2,3]
+                        
+        */
     
 private:
     
@@ -126,15 +142,16 @@ private:
     
     // matrix exp truncation precision
     
- 
+    bool        _validateCompressedStorage (void) const;
 
 public:
+    
 
     // constructors
 
     _Matrix ();                                 // default constructor, doesn't do much
 
-    _Matrix (_String const&, bool, _FormulaParsingContext&);
+    _Matrix (_String const&, bool, _FormulaParsingContext&, bool use_square_brackets = false);
     // matrix from a string of the form
     // {{i11,i12,...}{i21,i22,..}{in1,in2..)})
     // or {# rows,<# cols>{i1,j1,expr}{i2,j2,expr}..}
@@ -162,7 +179,7 @@ public:
 
     _Matrix ( _List const &, bool parse_escapes = true);                         //make string matrix from a list
 
-    _Matrix (hyFloat *, unsigned long, unsigned long);
+    _Matrix (const hyFloat *, unsigned long, unsigned long);
     /*
         20110616 SLKP
             added a simple constructor from a list of floating point values
@@ -179,9 +196,10 @@ public:
 
     ~_Matrix (void);                            //destructor
 
-    virtual void    Clear (void);               //deletes all the entries w/o destroying the matrix
+    virtual void    Clear (bool complete = true);               //deletes all the entries w/o destroying the matrix
+    virtual void    ZeroNumericMatrix (void);               //deletes all the entries w/o destroying the matrix
 
-    void    Initialize (bool = false);                  // zeros all matrix structures
+    void    Initialize (bool = true);                  // zeros all matrix structures
 
     virtual void        Serialize (_StringBuffer&,_String&);
     // write the matrix definition in HBL
@@ -198,19 +216,21 @@ public:
     inline bool        is_expression_based (void) const {return storageType == _FORMULA_TYPE;}
     inline bool        is_numeric (void) const {return storageType == _NUMERICAL_TYPE;}
     inline bool        is_polynomial (void) const {return storageType == _POLYNOMIAL_TYPE;}
+    inline bool        has_type (int t) const {return storageType == t;}
 
     HBLObjectRef           Evaluate (bool replace = true); // evaluates the matrix if contains formulas
     // if replace is true, overwrites the original
 
-    virtual HBLObjectRef   ExecuteSingleOp (long opCode, _List* arguments = nil, _hyExecutionContext* context = _hyDefaultExecutionContext);
+    virtual HBLObjectRef   ExecuteSingleOp (long opCode, _List* arguments = nil, _hyExecutionContext* context = _hyDefaultExecutionContext, HBLObjectRef cache = nil);
     // execute this operation with the list of Args
 
-    HBLObjectRef   MAccess (HBLObjectRef, HBLObjectRef);
+    HBLObjectRef   MAccess (HBLObjectRef, HBLObjectRef, HBLObjectRef cache = nil);
     // implements the M[i][j] operation for formulas
-    HBLObjectRef   MCoord (HBLObjectRef, HBLObjectRef);
+    HBLObjectRef   MCoord (HBLObjectRef, HBLObjectRef, HBLObjectRef cache = nil);
     // implements the M[i][j] operation for formulas
 
     void        MStore (long, long, _Formula&, long = -1);
+    void        MStore (long, long, HBLObjectRef, long);
     bool        MResolve (HBLObjectRef, HBLObjectRef, long&, long&);
     // resolve coordiates from two Number arguments
 
@@ -256,17 +276,17 @@ public:
     _Matrix const&     operator = (_Matrix const&);             // assignment operation on matrices
     _Matrix const&     operator = (_Matrix const*);             // assignment operation on matrices with temp results
 
-    virtual HBLObjectRef    Random (HBLObjectRef);    // reshuffle the matrix
+    virtual HBLObjectRef    Random (HBLObjectRef, HBLObjectRef cache);    // reshuffle the matrix
 
-    virtual HBLObjectRef    AddObj (HBLObjectRef);    // addition operation on matrices
+    virtual HBLObjectRef    AddObj (HBLObjectRef, HBLObjectRef cache);    // addition operation on matrices
 
-    virtual HBLObjectRef    SubObj (HBLObjectRef);    // subtraction operation on matrices
+    virtual HBLObjectRef    SubObj (HBLObjectRef, HBLObjectRef cache);    // subtraction operation on matrices
 
-    virtual HBLObjectRef    MultObj (HBLObjectRef);   // multiplication operation on matrices
+    virtual HBLObjectRef    MultObj (HBLObjectRef, HBLObjectRef cache);   // multiplication operation on matrices
 
-    virtual HBLObjectRef    MultElements (HBLObjectRef, bool elementWiseDivide = false);  // element wise multiplication/division operation on matrices
+    virtual HBLObjectRef    MultElements (HBLObjectRef, bool elementWiseDivide, HBLObjectRef cache);  // element wise multiplication/division operation on matrices
 
-    virtual HBLObjectRef    Sum          (void);
+    virtual HBLObjectRef    Sum          (HBLObjectRef cache);
 
     _Matrix     operator + (_Matrix&);          // addition operation on matrices
 
@@ -298,8 +318,8 @@ public:
     //               added a boolean flag to allow numeric matrices
     //               to be implicitly converted to strings
 
-    _Matrix*    NeighborJoin                    (bool);
-    _Matrix*    MakeTreeFromParent              (long);
+    _Matrix*    NeighborJoin                    (bool, HBLObjectRef cache);
+    _Matrix*    MakeTreeFromParent              (long, HBLObjectRef cache);
     _Matrix*    ExtractElementsByEnumeration    (_SimpleList*,_SimpleList*,bool=false);
     _Matrix*    SimplexSolve                    (hyFloat = 1.e-6);
 
@@ -308,22 +328,22 @@ public:
 // square the matrix by Strassen's Multiplication
 
 
-    _Matrix*    Exponentiate (hyFloat scale_to = 1.0, bool check_transition = false);                // exponent of a matrix
+    _Matrix*    Exponentiate (hyFloat scale_to = 0.5, bool check_transition = false, _Matrix * write_here = nil);                // exponent of a matrix
     void        Transpose (void);                   // transpose a matrix
     _Matrix     Gauss   (void);                     // Gaussian Triangularization process
     HBLObjectRef   LUDecompose (void) const;
     HBLObjectRef   CholeskyDecompose (void) const;
     // added by afyp July 6, 2009
-    HBLObjectRef   Eigensystem (void) const;
+    HBLObjectRef   Eigensystem (HBLObjectRef) const;
     HBLObjectRef   LUSolve (HBLObjectRef) const;
-    HBLObjectRef   Inverse (void) const;
-    HBLObjectRef   Abs (void);                     // returns the norm of a matrix
+    HBLObjectRef   Inverse (HBLObjectRef cache) const;
+    HBLObjectRef   Abs (HBLObjectRef cache);                     // returns the norm of a matrix
     // if it is a vector - returns the Euclidean length
     // otherwise returns the largest element
 
     hyFloat  AbsValue                        (void) const;
     
-    template <typename CALLBACK>  HBLObjectRef ApplyScalarOperation (CALLBACK && functor) const;
+    template <typename CALLBACK>  HBLObjectRef ApplyScalarOperation (CALLBACK && functor, HBLObjectRef cache) const;
     
     // return the matrix of logs of every matrix element
     
@@ -436,13 +456,14 @@ public:
             if the matrix is sparse, only will out the non-void entries
      */
 
-    _Formula*   GetFormula                  (long, long) const;
-    HBLObjectRef   MultByFreqs                 (long);
-    HBLObjectRef   EvaluateSimple              (void);
-    HBLObjectRef   SortMatrixOnColumn          (HBLObjectRef);
-    HBLObjectRef   K_Means                     (HBLObjectRef);
-    HBLObjectRef   pFDR                        (HBLObjectRef);    // positive false discovery rate
-    HBLObjectRef   PoissonLL                   (HBLObjectRef);    // log likelihood of a vector of poisson samples given a parameter value
+    _Formula*      GetFormula                  (long, long) const;
+    HBLObjectRef   GetMatrixCell               (long, long, HBLObjectRef cache = nil) const;
+    HBLObjectRef   MultByFreqs                 (long, bool = false);
+    HBLObjectRef   EvaluateSimple              (_Matrix* existing_receptacle = nil);
+    HBLObjectRef   SortMatrixOnColumn          (HBLObjectRef, HBLObjectRef cache);
+    HBLObjectRef   K_Means                     (HBLObjectRef, HBLObjectRef cache);
+    HBLObjectRef   pFDR                        (HBLObjectRef, HBLObjectRef cache);    // positive false discovery rate
+    HBLObjectRef   PoissonLL                   (HBLObjectRef, HBLObjectRef cache);    // log likelihood of a vector of poisson samples given a parameter value
 
 
     // added by afyp, July 1, 2009
@@ -497,9 +518,6 @@ public:
     long              MatrixType (void) {
         return storageType;
     }
-    bool              SparseDataStructure (void) {
-        return theIndex;
-    }
     
     template <typename CALLBACK, typename EXTRACTOR>  void ForEach (CALLBACK&& cbv, EXTRACTOR&& accessor) const {
         if (theIndex) {
@@ -552,7 +570,7 @@ public:
         return true;
     }
 
-    void              CheckIfSparseEnough (bool = false);       // check if matrix is sparse enough to justify
+    bool              CheckIfSparseEnough (bool = false, bool copy = true);       // check if matrix is sparse enough to justify
     void              Convert2Formulas      (void);     // converts a numeric matrix to formula-based mx
     // sparse storage
 
@@ -598,6 +616,7 @@ public:
 
     static void    CreateMatrix    (_Matrix* theMatrix, long theHDim, long theVDim,  bool sparse = false, bool allocateStorage = false, bool isFla = false);
 
+    void        RecursiveIndexSort      (long, long, _SimpleList*);
 
 
 
@@ -611,7 +630,7 @@ private:
     void        InitMxVar           (_SimpleList&   , hyFloat);
     bool        ProcessFormulas     (long&, _AVLList&, _SimpleList&, _SimpleList&, _AVLListX&, bool = false, _Matrix* = nil);
 
-    HBLObjectRef   PathLogLikelihood   (HBLObjectRef);
+    HBLObjectRef   PathLogLikelihood   (HBLObjectRef, HBLObjectRef cache);
     /* SLKP: 20100812
 
      This function assumes that 'this' an 3xK matrix, where each column is of the form
@@ -661,7 +680,6 @@ private:
 
 
     void        BreakPoints             (long, long, _SimpleList*);
-    void        RecursiveIndexSort      (long, long, _SimpleList*);
     void        ConvertFormulas2Poly    (bool force2numbers=true);
     void        ConvertNumbers2Poly     (void);
     void        AgreeObjects            (_Matrix&);
@@ -696,9 +714,10 @@ extern  _Matrix *GlobalFrequenciesMatrix;
 // the matrix of frequencies for the trees to be set by block likelihood evaluator
 extern  long  ANALYTIC_COMPUTATION_FLAG;
 
+HBLObjectRef _returnMatrixOrUseCache (long nrow, long ncol, long type, bool is_sparse, HBLObjectRef cache);
 
 #ifdef  _SLKP_USE_AVX_INTRINSICS
-    inline const double _avx_sum_4 (__m256d const & x) {
+    inline double _avx_sum_4 (__m256d const & x) {
       /*__m256d t = _mm256_add_pd (_mm256_shuffle_pd (x, x, 0x0),
                                  // (x3,x3,x1,x1)
                                  _mm256_shuffle_pd (x, x, 0xf)
@@ -710,6 +729,7 @@ extern  long  ANALYTIC_COMPUTATION_FLAG;
                                        ));
        */
         __m256d sum      = _mm256_hadd_pd(x, x);
+        // sum now has (x[0]+x[1],x[0]+x[1],x[2]+x[3],x[2]+x[3])
         return _mm_cvtsd_f64(_mm_add_pd(_mm256_extractf128_pd(sum, 1), _mm256_castpd256_pd128(sum)));
         /*double  __attribute__ ((aligned (32))) array[4];
         _mm256_store_pd (array, x);
