@@ -65,11 +65,11 @@ _Operation::_Operation  (void) {
  }
 
 //__________________________________________________________________________________
-void    _Operation::Initialize(bool)
-{
+void    _Operation::Initialize(bool) {
     numberOfTerms = 0;
     theData = -1;
     theNumber = nil;
+    cachedResult = nil;
     opCode = -1;
 }
 
@@ -87,12 +87,17 @@ _Operation::_Operation (_Operation const& rhs) {
     theData        = rhs.theData;
     theNumber      = rhs.theNumber;
     opCode         = rhs.opCode;
+    cachedResult   = rhs.cachedResult;
     if (theNumber) {
         theNumber->AddAReference();
     }
+    if (cachedResult) {
+        cachedResult->AddAReference();
+    }
 }
 
 
+
 //__________________________________________________________________________________
 void    _Operation::Duplicate(BaseRefConst r) {
     _Operation const * o = (_Operation const*)r;
@@ -100,9 +105,18 @@ void    _Operation::Duplicate(BaseRefConst r) {
     theData        = o->theData;
     theNumber      = o->theNumber;
     opCode         = o->opCode;
+    cachedResult   = o->cachedResult;
     if (theNumber) {
         theNumber->AddAReference();
     }
+    if (cachedResult) {
+        cachedResult->AddAReference();
+    }
+}
+
+//__________________________________________________________________________________
+void    _Operation::operator = (_Operation const & rhs) {
+    Duplicate (&rhs);
 }
 
 
@@ -129,13 +143,13 @@ BaseRef _Operation::toStr (unsigned long) {
 }
 
 //__________________________________________________________________________________
-_Operation::_Operation  (const long theCode, const long opNo = 2)
+_Operation::_Operation  (const long theCode, const long opNo = 2) {
 // by opcode
-{
     opCode = theCode;
     numberOfTerms = opNo;
     theData       = -1;
     theNumber     = nil;
+    cachedResult  = nil;
 }
 
 //__________________________________________________________________________________
@@ -156,6 +170,7 @@ _Operation::_Operation  (_String const& opc, const long opNo = 2) {
     numberOfTerms = opNo;
     theData       = -1;
     theNumber     = nil;
+    cachedResult  = nil;
 }
 //__________________________________________________________________________________
 _Operation::_Operation  (HBLObjectRef theObj) {
@@ -163,6 +178,7 @@ _Operation::_Operation  (HBLObjectRef theObj) {
     theData       = -1;
     opCode        = -1;
     theNumber     = theObj;
+    cachedResult = nil;
 }
 
 //__________________________________________________________________________________
@@ -171,11 +187,11 @@ _Operation::_Operation  (_Variable const & v) {
     theData       = v.get_index();
     opCode        = -1;
     theNumber     = nil;
+    cachedResult  = nil;
 }
 
 //__________________________________________________________________________________
-bool _Operation::CanResultsBeCached (_Operation* prev, bool exp_only)
-{
+bool _Operation::CanResultsBeCached (_Operation* prev, bool exp_only) {
     if (theNumber == nil && theData == -1 && numberOfTerms == 1) {
         if ((prev->theNumber && prev->theNumber->ObjectClass() == MATRIX)
            || (prev->theData >= 0 && LocateVar (prev->theData)->ObjectClass () == MATRIX)) {
@@ -198,8 +214,7 @@ bool   _Operation::IsConstantOfType   (const long type) const {
 
 //__________________________________________________________________________________
 
-bool _Operation::HasChanged (void)
-{
+bool _Operation::HasChanged (void) {
     if (theNumber) {
         return theNumber->HasChanged();
     }
@@ -249,17 +264,14 @@ _Operation::_Operation  (bool isVar, _String& stuff, bool isG, _VariableContaine
         theData = -1L;
     }
     opCode = -1L;
-
+    cachedResult = nil;
 }
 
 
 //__________________________________________________________________________________
-_Operation::~_Operation (void)
-{
-
-    if (theNumber) {
-        DeleteObject (theNumber);
-    }
+_Operation::~_Operation (void) {
+   DeleteAndZeroObject (theNumber);
+   DeleteAndZeroObject (cachedResult);
 }
 
 //__________________________________________________________________________________
@@ -271,7 +283,6 @@ bool _Operation::IsAVariable(bool deep) {
         return false;
     }
     return true;
-
 }
 
 //__________________________________________________________________________________
@@ -367,7 +378,7 @@ long        _Operation::StackDepth (void) const {
 
 
 //__________________________________________________________________________________
-bool        _Operation::Execute (_Stack& theScrap, _VariableContainer const* nameSpace, _String* errMsg) {
+bool        _Operation::Execute (_Stack& theScrap, _VariableContainer const* nameSpace, _String* errMsg, bool canCache) {
   if (theNumber) { // push value
     theScrap.Push(theNumber);
     return true;
@@ -419,7 +430,7 @@ bool        _Operation::Execute (_Stack& theScrap, _VariableContainer const* nam
         _String         *argument_k = (_String*)funcVarList->Element(k);
         HBLObjectRef       nthterm = theScrap.Pop();
 
-        //printf ("\tArgument %d : %s (%s)\n", k, argument_k->sData, _String((_String*)nthterm->toStr()).sData);
+        //printf ("\tArgument %d : %s (%s)\n", k, argument_k->get_str(), _String((_String*)nthterm->toStr()).get_str());
 
         if (isRefVar) {
           need_to_purge = true;
@@ -454,7 +465,7 @@ bool        _Operation::Execute (_Stack& theScrap, _VariableContainer const* nam
             existingIVars<<argument_var->get_index();
           } else {
             _Variable *newV = new _Variable (*argument_k);
-            newV->SetValue(nthterm,false);
+            newV->SetValue(nthterm,false,true, NULL);
             existingDVars<<argument_var->get_index();
             displacedVars<<argument_var; // 2 references
             argument_var->AddAReference(); // 3 references
@@ -586,11 +597,24 @@ bool        _Operation::Execute (_Stack& theScrap, _VariableContainer const* nam
   if (numberOfTerms > 1) {
     _List arguments;
 
-    for (long k = 1; k < numberOfTerms; k ++) {
-      arguments.AppendNewInstance ((HBLObjectRef)theScrap.theStack.list_data[theScrap.theStack.lLength - numberOfTerms + k]);
+    if (numberOfTerms == 2) {
+        arguments.AppendNewInstance ((HBLObjectRef)theScrap.theStack.list_data[theScrap.theStack.lLength - 1]);
+    } else {
+        for (long k = 1; k < numberOfTerms; k ++) {
+          arguments.AppendNewInstance ((HBLObjectRef)theScrap.theStack.list_data[theScrap.theStack.lLength - numberOfTerms + k]);
+        }
     }
     
-    temp =  arg0->ExecuteSingleOp(opCode, &arguments, &localContext);
+    if (canCache) {
+        temp =  arg0->ExecuteSingleOp(opCode, &arguments, &localContext, cachedResult);
+        if (temp != cachedResult) {
+            DeleteObject(cachedResult);
+            cachedResult = temp;
+        }
+        temp->AddAReference();
+    } else {
+        temp =  arg0->ExecuteSingleOp(opCode, &arguments, &localContext);
+    }
     theScrap.theStack.lLength -= (numberOfTerms);
 
   } else {
@@ -633,11 +657,15 @@ bool        _Operation::ExecutePolynomial (_Stack& theScrap, _VariableContainer*
 
     _Polynomial*p = nil;
     if (theNumber) {
-        p= new _Polynomial(theNumber->Value());
-    }
-
-    if (theData>-1) {
-        p=  new _Polynomial(*LocateVar(theData>-1?theData:-theData-2));
+        if (theNumber->ObjectClass() == NUMBER) {
+            p= new _Polynomial(theNumber->Value());
+        } else {
+            return false;
+        }
+    } else {
+        if (theData>-1) {
+            p=  new _Polynomial(*LocateVar(theData>-1?theData:-theData-2));
+        }
     }
 
     if (p) {
